@@ -3,12 +3,47 @@ This module handles compatibility issues between testcase format v2 and v3.
 """
 import os
 import sys
-from typing import List, Dict, Text, Union
+from typing import List, Dict, Text, Union, Any
 
 from loguru import logger
 
+from httprunner import exceptions
 from httprunner.loader import load_project_meta
-from httprunner.utils import sort_dict_by_custom_order
+from httprunner.parser import parse_data
+from httprunner.utils import sort_dict_by_custom_order, ensure_file_path_valid
+
+
+def convert_variables(
+    raw_variables: Union[Dict, List, Text], test_path: Text
+) -> Dict[Text, Any]:
+
+    if isinstance(raw_variables, Dict):
+        return raw_variables
+
+    if isinstance(raw_variables, List):
+        # [{"var1": 1}, {"var2": 2}]
+        variables: Dict[Text, Any] = {}
+        for var_item in raw_variables:
+            if not isinstance(var_item, Dict) or len(var_item) != 1:
+                raise exceptions.TestCaseFormatError(
+                    f"Invalid variables format: {raw_variables}"
+                )
+
+            variables.update(var_item)
+
+        return variables
+
+    elif isinstance(raw_variables, Text):
+        # get variables by function, e.g. ${get_variables()}
+        project_meta = load_project_meta(test_path)
+        variables = parse_data(raw_variables, {}, project_meta.functions)
+
+        return variables
+
+    else:
+        raise exceptions.TestCaseFormatError(
+            f"Invalid variables format: {raw_variables}"
+        )
 
 
 def convert_jmespath(raw: Text) -> Text:
@@ -53,10 +88,12 @@ def convert_extractors(extractors: Union[List, Dict]) -> Dict:
     v3_extractors: Dict = {}
 
     if isinstance(extractors, List):
+        # [{"varA": "content.varA"}, {"varB": "json.varB"}]
         for extractor in extractors:
             for k, v in extractor.items():
                 v3_extractors[k] = v
     elif isinstance(extractors, Dict):
+        # {"varA": "body.varA", "varB": "body.varB"}
         v3_extractors = extractors
     else:
         logger.error(f"Invalid extractor: {extractors}")
@@ -133,10 +170,10 @@ def ensure_step_attachment(step: Dict) -> Dict:
         test_dict["teardown_hooks"] = step["teardown_hooks"]
 
     if "extract" in step:
-        if step.get("request"):
-            test_dict["extract"] = convert_extractors(step["extract"])
-        elif step.get("testcase"):
-            test_dict["extract"] = step["extract"]
+        test_dict["extract"] = convert_extractors(step["extract"])
+
+    if "export" in step:
+        test_dict["export"] = step["export"]
 
     if "validate" in step:
         test_dict["validate"] = convert_validators(step["validate"])
@@ -167,14 +204,16 @@ def ensure_testcase_v3(test_content: Dict) -> Dict:
     for step in test_content["teststeps"]:
         teststep = {}
 
-        teststep.update(ensure_step_attachment(step))
-
         if "request" in step:
             teststep["request"] = step.pop("request")
         elif "api" in step:
             teststep["testcase"] = step.pop("api")
         elif "testcase" in step:
             teststep["testcase"] = step.pop("testcase")
+        else:
+            raise exceptions.TestCaseFormatError(f"Invalid teststep: {step}")
+
+        teststep.update(ensure_step_attachment(step))
 
         teststep = sort_step_by_custom_order(teststep)
         v3_content["teststeps"].append(teststep)
@@ -215,10 +254,8 @@ def generate_conftest_for_summary(args: List):
         sys.exit(1)
 
     project_meta = load_project_meta(test_path)
-    conftest_path = os.path.join(project_meta.PWD, "conftest.py")
-    if os.path.isfile(conftest_path):
-        return
-
+    project_root_dir = ensure_file_path_valid(project_meta.RootDir)
+    conftest_path = os.path.join(project_root_dir, "conftest.py")
     conftest_content = '''# NOTICE: Generated By HttpRunner.
 import json
 import os
@@ -227,7 +264,7 @@ import time
 import pytest
 from loguru import logger
 
-from httprunner.utils import get_platform
+from httprunner.utils import get_platform, ExtendJSONEncoder
 
 
 @pytest.fixture(scope="session", autouse=True)
@@ -279,15 +316,15 @@ def session_fixture(request):
     os.makedirs(summary_dir, exist_ok=True)
 
     with open(summary_path, "w", encoding="utf-8") as f:
-        json.dump(summary, f, indent=4)
+        json.dump(summary, f, indent=4, ensure_ascii=False, cls=ExtendJSONEncoder)
 
     logger.info(f"generated task summary: {summary_path}")
 
 '''
 
     test_path = os.path.abspath(test_path)
-    logs_dir_path = os.path.join(project_meta.PWD, "logs")
-    test_path_relative_path = test_path[len(project_meta.PWD) + 1 :]
+    logs_dir_path = os.path.join(project_root_dir, "logs")
+    test_path_relative_path = test_path[len(project_root_dir) + 1 :]
 
     if os.path.isdir(test_path):
         file_foder_path = os.path.join(logs_dir_path, test_path_relative_path)
@@ -303,6 +340,10 @@ def session_fixture(request):
         "{{SUMMARY_PATH_PLACEHOLDER}}", summary_path
     )
 
+    dir_path = os.path.dirname(conftest_path)
+    if not os.path.exists(dir_path):
+        os.makedirs(dir_path)
+
     with open(conftest_path, "w", encoding="utf-8") as f:
         f.write(conftest_content)
 
