@@ -2,7 +2,7 @@ import os
 import time
 import uuid
 from datetime import datetime
-from typing import List, Dict, Text, NoReturn
+from typing import List, Dict, Text, NoReturn, Union
 
 try:
     import allure
@@ -21,7 +21,7 @@ from httprunner.loader import load_project_meta, load_testcase_file
 from httprunner.parser import build_url, parse_data, parse_variables_mapping
 from httprunner.response import ResponseObject
 from httprunner.testcase import Config, Step
-from httprunner.schema import (
+from httprunner.models import (
     TConfig,
     TStep,
     VariablesMapping,
@@ -31,6 +31,7 @@ from httprunner.schema import (
     TestCaseInOut,
     ProjectMeta,
     TestCase,
+    Hooks,
 )
 
 
@@ -43,10 +44,10 @@ class HttpRunner(object):
     __teststeps: List[TStep]
     __project_meta: ProjectMeta = None
     __case_id: Text = ""
-    __step_datas: List[StepData] = None
+    __export: List[Text] = []
+    __step_datas: List[StepData] = []
     __session: HttpSession = None
     __session_variables: VariablesMapping = {}
-    __export_variables: VariablesMapping = {}
     # time
     __start_at: float = 0
     __duration: float = 0
@@ -82,6 +83,56 @@ class HttpRunner(object):
         self.__session_variables = variables
         return self
 
+    def with_export(self, export: List[Text]) -> "HttpRunner":
+        self.__export = export
+        return self
+
+    def __call_hooks(
+        self, hooks: Hooks, step_variables: VariablesMapping, hook_type: Text,
+    ) -> NoReturn:
+        """ call hook actions.
+
+        Args:
+            hooks (list): each hook in hooks list maybe in two format.
+
+                format1 (str): only call hook functions.
+                    ${func()}
+                format2 (dict): assignment, the value returned by hook function will be assigned to variable.
+                    {"var": "${func()}"}
+
+            step_variables: current step variables to call hook, include two special variables
+
+                request: parsed request dict
+                response: ResponseObject for current response
+
+            hook_type: setup/teardown
+
+        """
+        logger.debug(f"call {hook_type} hook actions.")
+
+        if not isinstance(hooks, List):
+            logger.error(f"Invalid hooks format: {hooks}")
+            return
+
+        for hook in hooks:
+            if isinstance(hook, Text):
+                # format 1: ["${func()}"]
+                logger.debug(f"call hook function: {hook}")
+                parse_data(hook, step_variables, self.__project_meta.functions)
+            elif isinstance(hook, Dict) and len(hook) == 1:
+                # format 2: {"var": "${func()}"}
+                var_name, hook_content = list(hook.items())[0]
+                hook_content_eval = parse_data(
+                    hook_content, step_variables, self.__project_meta.functions
+                )
+                logger.debug(
+                    f"call hook function: {hook_content}, got value: {hook_content_eval}"
+                )
+                logger.debug(f"assign variable: {var_name} = {hook_content_eval}")
+                step_variables[var_name] = hook_content_eval
+            else:
+                logger.error(f"Invalid hook format: {hook}")
+
     def __run_step_request(self, step: TStep) -> StepData:
         """run teststep: request"""
         step_data = StepData(name=step.name)
@@ -97,6 +148,11 @@ class HttpRunner(object):
             "HRUN-Request-ID",
             f"HRUN-{self.__case_id}-{str(int(time.time() * 1000))[-6:]}",
         )
+        step.variables["request"] = parsed_request_dict
+
+        # setup hooks
+        if step.setup_hooks:
+            self.__call_hooks(step.setup_hooks, step.variables, "setup")
 
         # prepare arguments
         method = parsed_request_dict.pop("method")
@@ -108,6 +164,11 @@ class HttpRunner(object):
         # request
         resp = self.__session.request(method, url, **parsed_request_dict)
         resp_obj = ResponseObject(resp)
+        step.variables["response"] = resp_obj
+
+        # teardown hooks
+        if step.teardown_hooks:
+            self.__call_hooks(step.teardown_hooks, step.variables, "teardown")
 
         def log_req_resp_details():
             err_msg = "\n{} DETAILED REQUEST & RESPONSE {}\n".format("*" * 32, "*" * 32)
@@ -134,7 +195,7 @@ class HttpRunner(object):
         # extract
         extractors = step.extract
         extract_mapping = resp_obj.extract(extractors)
-        step_data.export = extract_mapping
+        step_data.export_vars = extract_mapping
 
         variables_mapping = step.variables
         variables_mapping.update(extract_mapping)
@@ -166,6 +227,7 @@ class HttpRunner(object):
         """run teststep: referenced testcase"""
         step_data = StepData(name=step.name)
         step_variables = step.variables
+        step_export = step.export
 
         if hasattr(step.testcase, "config") and hasattr(step.testcase, "teststeps"):
             testcase_cls = step.testcase
@@ -174,6 +236,7 @@ class HttpRunner(object):
                 .with_session(self.__session)
                 .with_case_id(self.__case_id)
                 .with_variables(step_variables)
+                .with_export(step_export)
                 .run()
             )
 
@@ -181,13 +244,16 @@ class HttpRunner(object):
             if os.path.isabs(step.testcase):
                 ref_testcase_path = step.testcase
             else:
-                ref_testcase_path = os.path.join(self.__project_meta.PWD, step.testcase)
+                ref_testcase_path = os.path.join(
+                    self.__project_meta.RootDir, step.testcase
+                )
 
             case_result = (
                 HttpRunner()
                 .with_session(self.__session)
                 .with_case_id(self.__case_id)
                 .with_variables(step_variables)
+                .with_export(step_export)
                 .run_path(ref_testcase_path)
             )
 
@@ -197,10 +263,13 @@ class HttpRunner(object):
             )
 
         step_data.data = case_result.get_step_datas()  # list of step data
-        step_data.export = case_result.get_export_variables()
+        step_data.export_vars = case_result.get_export_variables()
         step_data.success = case_result.success
         self.success &= case_result.success
 
+        if step_data.export_vars:
+            logger.info(f"export variables: {step_data.export_vars}")
+
         return step_data
 
     def __run_step(self, step: TStep) -> Dict:
@@ -218,7 +287,7 @@ class HttpRunner(object):
 
         self.__step_datas.append(step_data)
         logger.info(f"run step end: {step.name} <<<<<<\n")
-        return step_data.export
+        return step_data.export_vars
 
     def __parse_config(self, config: TConfig) -> NoReturn:
         config.variables.update(self.__session_variables)
@@ -252,7 +321,6 @@ class HttpRunner(object):
         self.__step_datas: List[StepData] = []
         self.__session = self.__session or HttpSession()
         self.__session_variables = {}
-        self.__export_variables = {}
 
         # run teststeps
         for step in self.__teststeps:
@@ -274,11 +342,6 @@ class HttpRunner(object):
             self.__session_variables.update(extract_mapping)
 
         self.__duration = time.time() - self.__start_at
-
-        self.__export_variables = self.get_export_variables()
-        if self.__export_variables:
-            logger.info(f"export variables: {self.__export_variables}")
-
         return self
 
     def run_path(self, path: Text) -> "HttpRunner":
@@ -303,11 +366,10 @@ class HttpRunner(object):
         return self.__step_datas
 
     def get_export_variables(self) -> Dict:
-        if self.__export_variables:
-            return self.__export_variables
-
+        # override testcase export vars with step export
+        export_var_names = self.__export or self.__config.export
         export_vars_mapping = {}
-        for var_name in self.__config.export:
+        for var_name in export_var_names:
             if var_name not in self.__session_variables:
                 raise ParamsError(
                     f"failed to export variable {var_name} from session variables {self.__session_variables}"
@@ -331,7 +393,8 @@ class HttpRunner(object):
                 duration=self.__duration,
             ),
             in_out=TestCaseInOut(
-                vars=self.__config.variables, export=self.get_export_variables()
+                config_vars=self.__config.variables,
+                export_vars=self.get_export_variables(),
             ),
             log=self.__log_path,
             step_datas=self.__step_datas,
@@ -345,7 +408,7 @@ class HttpRunner(object):
         )
         self.__case_id = self.__case_id or str(uuid.uuid4())
         self.__log_path = self.__log_path or os.path.join(
-            self.__project_meta.PWD, "logs", f"{self.__case_id}.run.log"
+            self.__project_meta.RootDir, "logs", f"{self.__case_id}.run.log"
         )
         log_handler = logger.add(self.__log_path, level="DEBUG")
 
